@@ -6,6 +6,9 @@ else
     gjs="$LOG_COMPILER $LOG_FLAGS gjs-console"
 fi
 
+# Avoid interference in the profiler tests from stray environment variable
+unset GJS_ENABLE_PROFILER
+
 # This JS script should exit immediately with code 42. If that is not working,
 # then it will exit after 3 seconds as a fallback, with code 0.
 cat <<EOF >exit.js
@@ -72,6 +75,11 @@ report_xfail () {
     fi
 }
 
+skip () {
+    total=$((total + 1))
+    echo "ok $total - $1 # SKIP $2"
+}
+
 # Test that System.exit() works in gjs-console
 $gjs -c 'imports.system.exit(0)'
 report "System.exit(0) should exit successfully"
@@ -113,14 +121,14 @@ report "--help should print something before -c"
 # --help after a script file name is passed to the script
 $gjs -I sentinel help.js --help
 report "--help after script file should be passed to script"
-test -z "$("$gjs" -I sentinel help.js --help)"
+test -z "$($gjs -I sentinel help.js --help)"
 report "--help after script file should not print anything"
 
 # --help after a -c argument is passed to the script
 script='if(ARGV[0] !== "--help") imports.system.exit(1)'
 $gjs -c "$script" --help
 report "--help after -c should be passed to script"
-test -z "$("$gjs" -c "$script" --help)"
+test -z "$($gjs -c "$script" --help)"
 report "--help after -c should not print anything"
 
 # -I after a program is not consumed by GJS
@@ -135,6 +143,9 @@ report "--coverage-prefix after script should succeed but give a warning"
 $gjs -c 'imports.system.exit(0)' --coverage-prefix=foo --coverage-output=foo 2>&1 | grep -q 'Gjs-WARNING.*--coverage-output'
 report "--coverage-output after script should succeed but give a warning"
 rm -f foo/coverage.lcov
+$gjs -c 'imports.system.exit(0)' --profile=foo 2>&1 | grep -q 'Gjs-WARNING.*--profile'
+report "--profile after script should succeed but give a warning"
+rm -rf foo
 
 # --version works
 $gjs --version >/dev/null
@@ -149,6 +160,30 @@ report "--version after -c should be passed to script"
 test -z "$($gjs -c "$script" --version)"
 report "--version after -c should not print anything"
 
+# --profile
+rm -f gjs-*.syscap foo.syscap
+$gjs -c 'imports.system.exit(0)' && ! stat gjs-*.syscap &> /dev/null
+report "no profiling data should be dumped without --profile"
+
+# Skip some tests if built without profiler support
+if $gjs --profile -c 1 2>&1 | grep -q 'Gjs-Message.*Profiler is disabled'; then
+    reason="profiler is disabled"
+    skip "--profile should dump profiling data to the default file name" "$reason"
+    skip "--profile with argument should dump profiling data to the named file" "$reason"
+    skip "GJS_ENABLE_PROFILER=1 should enable the profiler" "$reason"
+else
+    rm -f gjs-*.syscap
+    $gjs --profile -c 'imports.system.exit(0)' && stat gjs-*.syscap &> /dev/null
+    report "--profile should dump profiling data to the default file name"
+    rm -f gjs-*.syscap
+    $gjs --profile=foo.syscap -c 'imports.system.exit(0)' && test -f foo.syscap
+    report "--profile with argument should dump profiling data to the named file"
+    rm -f foo.syscap && rm -f gjs-*.syscap
+    GJS_ENABLE_PROFILER=1 $gjs -c 'imports.system.exit(0)' && stat gjs-*.syscap &> /dev/null
+    report "GJS_ENABLE_PROFILER=1 should enable the profiler"
+    rm -f gjs-*.syscap
+fi
+
 # interpreter handles queued promise jobs correctly
 output=$($gjs promise.js)
 test $? -eq 42
@@ -160,7 +195,7 @@ report "interpreter should stop running jobs when one calls System.exit()"
 
 $gjs -c "Promise.resolve().then(() => { throw new Error(); });" 2>&1 | grep -q 'Gjs-WARNING.*Unhandled promise rejection.*[sS]tack trace'
 report "unhandled promise rejection should be reported"
-test -z $($gjs awaitcatch.js)
+test -z "$($gjs awaitcatch.js)"
 report "catching an await expression should not cause unhandled rejection"
 
 rm -f exit.js help.js promise.js awaitcatch.js
